;
;	Implementation of XTEA (new varient of TEA) in 32 bit x86 assembly language
;
;	The code is a direct translation of ANSI C version, and commented with
;	pieces of the original C code for those interested, and hence should be 
;	straightforward. The code is not supposed to be optimized, but I think little 
;	(if anything) can be done to optimize it. It is much simpler than 16 bit
;	version, as the algorithm is designed for 32 bit CPUs, and there are enough
;	registers to avoid using stack or temporary memories.
;
;	Initially used as inline code in a VC++ program and then modified for use 
;	with MASM, but it should be very easy to modify it for other assemblers.
;
;	Placed in public domain by Farshid Mossaiby (mossaiby@yahoo.com)
;	Please feel free to send me comments or bugs.
;
;	To use the routines, simple fill in v and k arrays and call appropriate
;	routine.
;

.486
.MODEL FLAT, STDCALL
option        casemap:none

;include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\advapi32.inc
include \masm32\include\user32.inc
include \masm32\include\masm32.inc

includelib              \masm32\lib\kernel32.lib
includelib              \masm32\lib\advapi32.lib
includelib              \masm32\lib\user32.lib
includelib              \masm32\lib\masm32.lib


Main PROTO
; XTEA_Encipher PROTO
; XTEA_Decipher PROTO

;;		PUBLIC	v, w, k, XTEA_Encipher, XTEA_Decipher

; ..........................................................................
.const

Delta   equ     09E3779B9h      ;  Delta
DIS     equ     0C6EF3720h      ;  Initial value of sum in decipher routine
N       equ     32              ;  n (number of iterations) ... use it or not???

; ..........................................................................
; ..........................................................................

.data

v dd 0, 0               ;  Data to be encrypted or decrypted
w dd ?, ?               ;  Results
k dd 0,0,0,0            ;  Key

Password    db      "1234567890123456"  ; Zero Teminated or not

PlainText   db      "bbbbbbbbhhhhhhhh", 0


.code
Start:

XTEA_Encipher	proc

		PUSHAD					; Save registers

		MOV	EAX, v[0 * 4]		; EAX -> y
		MOV	EBX, v[1 * 4]		; EBX -> z
		MOV	ECX, N				; ECX -> n
		XOR	EDX, EDX			; EDX -> sum

		EnStart:

		MOV	ESI, EBX
		MOV	EDI, ESI

		SHL	ESI, 4				; z << 4
		SHR	EDI, 5				; z >> 5
		XOR	EDI, ESI			; z << 4 ^ z >> 5
		ADD	EDI, EBX			; (z << 4 ^ z >> 5) + z

		MOV	ESI, EDX
		AND	ESI, 3				; sum & 3
		MOV	ESI, k[ESI * 4]		; k[sum & 3]
		ADD	ESI, EDX			; sum + k[sum & 3]
		XOR	ESI, EDI			; (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3]
		ADD	EAX, ESI			; y += (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3]

		ADD	EDX, Delta			; sum += delta

		MOV	ESI, EAX
		MOV	EDI, ESI

		SHL	ESI, 4				; y << 4
		SHR	EDI, 5				; y >> 5
		XOR	EDI, ESI			; y << 4 ^ y >> 5
		ADD	EDI, EAX			; (y << 4 ^ y >> 5) + y

		MOV	ESI, EDX
		SHR	ESI, 11				; sum >> 11
		AND	ESI, 3				; sum >> 11 & 3
		MOV	ESI, k[ESI * 4]		; k[sum >> 11 & 3]
		ADD	ESI, EDX			; sum + k[sum >> 11 & 3]
		XOR	ESI, EDI			; (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3];
		ADD	EBX, ESI			; z += (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3];

		LOOP	EnStart

		MOV	w[0 * 4], EAX
		MOV	w[1 * 4], EBX

		POPAD					; Restore registers
		RET

XTEA_Encipher	endp

XTEA_Decipher	proc

		PUSHAD					; Save registers

		MOV	EAX, v[0 * 4]		; EAX -> y
		MOV	EBX, v[1 * 4]		; EBX -> z
		MOV	ECX, N				; ECX -> n
		MOV	EDX, DIS			; EDX -> sum

		DeStart:

		MOV	ESI, EAX
		MOV	EDI, ESI

		SHL	ESI, 4				; y << 4
		SHR	EDI, 5				; y >> 5
		XOR	EDI, ESI			; y << 4 ^ y >> 5
		ADD	EDI, EAX			; (y << 4 ^ y >> 5) + y

		MOV	ESI, EDX
		SHR	ESI, 11				; sum >> 11
		AND	ESI, 3				; sum >> 11 & 3
		MOV	ESI, k[ESI * 4]		; k[sum >> 11 & 3]
		ADD	ESI, EDX			; sum + k[sum >> 11 & 3]
		XOR	ESI, EDI			; (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3]
		SUB	EBX, ESI			; z -= (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3]

		SUB	EDX, Delta			; sum -= delta

		MOV	ESI, EBX
		MOV	EDI, ESI

		SHL	ESI, 4				; z << 4
		SHR	EDI, 5				; z >> 5
		XOR	EDI, ESI			; z << 4 ^ z >> 5
		ADD	EDI, EBX			; (z << 4 ^ z >> 5) + z

		MOV	ESI, EDX
		AND	ESI, 3				; sum & 3
		MOV	ESI, k[ESI * 4]		; k[sum & 3]
		ADD	ESI, EDX			; sum + k[sum & 3]
		XOR	ESI, EDI			; (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3]
		SUB	EAX, ESI			; y -= (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3]

		LOOP	DeStart

		MOV	w[0 * 4], EAX
		MOV	w[1 * 4], EBX

		POPAD					; Restore registers
		RET

XTEA_Decipher	endp




end Start